"""
Prompt templates — re-exported from focused submodules.
"""

from .prompts_analysis import ANALYSIS_PROMPT, AUDIO_ANALYSIS_PROMPT  # noqa: F401
from .prompts_music import MUSIC_BRIEF_ADDENDUM  # noqa: F401
from .prompts_edit import (  # noqa: F401
    EDIT_PROMPT_TEMPLATE,
    MARKER_EDIT_PROMPT_TEMPLATE,
    TIMELINE_CRITIQUE_PROMPT_TEMPLATE,
)
